#!/usr/bin/perl -w
#============================================================================
#
#  Suprnova Light mirror script
#
#----------------------------------------------------------------------------
#  by Weigon, zoo, isotopp, friedolin
#  License: GNU General Public License (http://www.gnu.org/copyleft/gpl.html)
#       
#  this script fetches the latest suprnova.org homepage and builds
#  a clean and functional webpage out of it.
# 
#  fetch the latest version from: http://www.rootdir.de/bt/
#  send patches and comments to: usenet@rootdir.de
#  								2004-09-05
#============================================================================
use strict;
use LWP;           
use HTTP::Status;
use HTTP::Request;
use LWP::UserAgent;
use POSIX 'strftime';
use File::Copy;
use Time::HiRes qw(gettimeofday tv_interval);
use URI::Escape;

#============================================================================
# Config the script here:
#============================================================================

    # give the main rotation website URI
    my $suprnova = 'http://www.suprnova.org/';

    # where the script lives
    # use an absolute path or use "." for current directory
    my $save_path = '/home/claas/www.rootdir.de/bt';
	
    # where to put the html-files
    my $htdocs = "/home/claas/www.rootdir.de/BT";

    # if you want to include a banner, put your banner code here
    my $banner='<p class="center">listen to <a href="http://media.hiof.no/streams/m3u/nrk-mpetre-128.m3u"><img src="http://media.hiof.no/images/mp3_logo.gif" alt="banner" /></a></p>';
    #my $banner='';

    # local proxy url
    #my $proxy='http://localhost:3128/';
    my $proxy='';

#============================================================================
# You do not need to edit anything below here!
#============================================================================

#====  MAIN  ================================================================
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
my %categorylist;
my $filename;

die "\$save_path '$save_path' does not exist!" if (! -d $save_path);
die "\$htdocs '$htdocs' does not exist!" if (! -d $htdocs);
die "$save_path/snmirror.mirrors does not exist!"
	if (! -f "$save_path/snmirror.mirrors");

#---- choose a mirror -------------------------------------------------------
my $mirror;
my $news;
if (!$ARGV[0]) {	#----- get random mirror ----------------------------
	$mirror = get($suprnova);
	if ( $mirror =~ /<frame name='main' src=['"]?([^'">]+)['"]?/sg ) {
	    print "Mirroring suprnova from $1 (random mirror)\n";
	    $mirror = $1;
	    my $found=0;
	    $filename=sprintf("%s/snmirror.mirrors", $save_path);
	    open (IN, "< $filename") || die "cannot open '$filename': $!";
	    open (OUT, "> $filename.tmp") || die "cannot open '$filename.tmp': $!";
	    while (<IN>) {
	    	chomp (my $server = $_);
			print OUT $server, "\n";
			if ($mirror eq $server) { $found=1; }
	    }
	    if ($found==0) { print OUT $mirror, "\n"; }
	    close IN;
	    close OUT;
	    move ($filename.".tmp", $filename);
	} else {	#----- use well known mirror ------------------------
        $mirror = backup_mirror();
	    print   "--> Using the fastest and latest mirror: $mirror\n";
	}
} else {	#----- use manual mirror ------------------------------------
	print "Mirroring suprnova from $ARGV[0] (manual mirror)\n";
	$mirror = $ARGV[0];
}

#---- get first page from that mirror ---------------------------------------
$news = get($mirror);
if (length($news) < 10000) {
    # fall back to backup mirror if random mirror sends too less data
    print "$mirror is unfunctional.\n";
    $mirror = backup_mirror();
    $news = get($mirror);
}

#---- save referer for later usage ------------------------------------------
open (OUT, "> $htdocs/referer.inc");
print OUT '<?php $referer="'.$mirror.'"; ?>';
close (OUT);

#---- parse news page -------------------------------------------------------
parse_contents ($news, "$htdocs/new", 'new');

#---- get and parse subpages ------------------------------------------------
while ( $news =~ /<a href=['"]?(http:\/\/[^'">]+\/list_torrents\/([^'">]+).html)['">]?/ig ) {
    my $category_name = $2;
    my $category_content = get($1);
    while ( $category_content =~ /<a href="JavaScript:showTbl\('([^']*\/(\d+))','([^']*)'\)/ig ) {
        my $subcat_url = $1.'.html';
        my $subcat_id =  $2;
        # $3 = short subcategory name
        $filename=sprintf('%s/%03d',$htdocs, $subcat_id);
        parse_sn ($subcat_url, $filename, $subcat_id);
    }
}

#---- build navigation line -------------------------------------------------
print "(idx) Building main Index\n";
my $lastgroup='none';
my $count=1;          # one might try with 1 plus the entries with "more:"
foreach my $cat (sort keys %categorylist) {
    my $catid=sprintf('%03d',$categorylist{$cat});
    if ($cat =~ /^(.*?) - (.*)$/) {
        my $subgroup=$2;
        my $group=$1;
        if ($group ne $lastgroup) { $count=$count+2; }
        $count++;
        $lastgroup = $group;
    }
}

$filename = "$htdocs/index";
write_header ($filename);
open (OUT, ">> $filename.html") || die "cannot open '$filename' for appending: $!";

my $catbreak=int($count/5)+3;
#print "catbreak $catbreak \n";
$count=1;
printf OUT '<table class="news"><tr><td><b><a href="new.html">New Torrents</a></b><br /> ';
foreach my $cat (sort keys %categorylist) {
    my $catid=sprintf('%03d',$categorylist{$cat});
    if ($cat =~ /^(.*?) - (.*)$/) {
        my $subgroup=$2;
        my $group=$1;
        if ($count >= $catbreak) {
	    	if ($lastgroup eq $group) {
	    	    printf OUT '</td><td><b>%s (more):</b><br />', $group;
		    } else {
	    	    printf OUT '</td><td><b>%s:</b><br />', $group;
    		}
	    	printf OUT '<a href="%s.html">%.25s</a><br />', $catid, $subgroup;
		    $count=2;
        } elsif ($count >= $catbreak-2) {
    		if ($lastgroup ne $group) {
	    		printf OUT '</td><td><b>%s:</b><br />', $group;
	    	    printf OUT '<a href="%s.html">%.25s</a><br />', $catid, $subgroup;
		    	$count=1;
    		} else {
		        printf OUT '<a href="%s.html">%.25s</a><br />', $catid, $subgroup;
	    		$count++;
    		}
	    } else {
		    if ($lastgroup ne $group) {
    			printf OUT '<br /><b>%s:</b><br />', $group;
	   	        $count=$count+2;
	   	    }
    		printf OUT '<a href="%s.html">%.25s</a><br />', $catid, $subgroup;
    		$count++;
	    }
        $lastgroup = $group;
    }
}
print OUT '</td></tr></table></body></html>';
close (OUT);

print "All done. - Get the latest snmirror.pl from http://www.rootdir.de/bt/ \n";

#====  SUB-ROUTINES  ========================================================

sub backup_mirror {
    my $fastest=0;
    my $fastest_url;
    my $lastage=0;
    my $mi;
    print "Checking known mirrors...\n";
    my $filename="$save_path/snmirror.mirrors";
    open (IN, "< $filename") || die "cannot open '$filename': $!";
    open (OUT, "> $filename.tmp") || die "cannot open '$filename.tmp': $!";
    while (<IN>) {
        chomp ($mi = $_);
        my $t0 = [gettimeofday];
            $news = get($mi."/list_news.html");
        my $t0_t1 = tv_interval $t0;
        my $speed=length($news)/$t0_t1/1024;
        $news =~ /Generated @ (\d{2})-(\d{2})-(\d{4}) \d{2}:\d{2} /;
        my $age=$3.$2.$1;
        if ($age >= $lastage && length($news) > 10000) {
            if ($speed > $fastest) {
                $fastest=$speed;
                $fastest_url=$mi;
            }
            printf "%6d Bytes% 8.2f KB/s %8d %s\n", length($news), $speed, $age, $mi;
            print OUT $mi, "\n";
            $lastage=$age;
        } else {
            printf "%6d Bytes% 8.2f KB/s %8d %s (bad mirror)\n", length($news), $speed, $age, $mi;
        }
    }
    close IN;
    close OUT;
    move ($filename.".tmp", $filename);
	print "mirror: $fastest_url\n";
    return $fastest_url;
}

#----- parse sn -------------------------------------------------------------
sub parse_sn {
    my $uri  = shift || return -1;
    my $filename = shift || return -1;
    my $subcat_id = shift || return -1;
    
    #----- download subcontents -----
    my $content = get($uri);
    parse_contents ( $content, $filename, $subcat_id);
}

#----- parse contents -------------------------------------------------------
sub parse_contents {
    my $content  = shift || return -1;
    my $filename = shift || return -1;
    my $subcat_id = shift || return -1;
    my %servers;
    my %sections;
    
    #----- get servers -----
    while ($content =~ /torrent_servers\[([-0-9]+?)\] = '(.+?)'/isg) {
        $servers{$1} = $2;
    }
    
    #----- get real contents -----
    while ($content =~ /(at[\d]?\(([0-9]+),'([^']*)',([0-9]+),([0-9]+),'([^']*)','([^']*)','([^']*)','([^']*)','([^']*)',([-0-9]+),([-0-9]+),'([^']*)','([^']*)','([^']*)','([^']*)','([^']*)',([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+))/isg ) {
        my @nova;
        my $line = $1;      # whole at()- or at2()-line
        $nova [5] = $7;     # Section
        $nova [1] = $3;     # time/date

        #----- prepare date for sorting -----
        my $xdate = sprintf("%02d-%02d", $mon+1, $mday);
        my $xyear = $year + 1900;
        $nova[1] =~ s/^(\d{2})-(\d{2})$/$2-$1/;
        if ($nova[1] =~ /-/) {
           if ($nova[1] le $xdate) {
              $nova[1] = "$xyear-" . $nova[1];
           } else {
              $nova[1] = $xyear-1 . "-" . $nova[1];
           }
        }
        
        #----- prepare timne for sorting -----
        if ($nova[1] =~ /([0-9]{2}):/) {
           my $h = $1;
           if ($h lt $hour) {
              $nova[1] = "9".$nova[1];
           }
        }
        push @{ $sections{$nova[5]}{$nova[1]} }, $line;
    }
   
    write_header ($filename);
    open(OUT,">> $filename.html") || die ("cannot open $filename for writing: $!");                         
   
    my $lgroup='';
    my $jumpto='';
    my $jumpcount=0;
    foreach my $section (sort keys %sections) {
        my $j;
        my $time;
    	if ($section =~ /^(.*?) - (.*)$/) {
	        my $group=$1;
	        if ($group ne $lgroup) { 
        		$jumpto = sprintf ("%s, <a href='#%s'>%s</a>",$jumpto, noblanks($group), $group);
			$jumpcount++;
		}
	        $lgroup = $group;
	}
    }
              
    my $count=0;
    my $previous_cat='none';
    #----- output sections -----
    foreach my $section (sort keys %sections) {
        my $j;
        my $time; 
	my $prev_catshort='none';
        #----- sort by time -----
        foreach $time (reverse sort keys %{ $sections{$section} }) {
            foreach $j ( 0 .. $#{ $sections{$section}{$time} } ) {
                $_ = $sections{$section}{$time}[$j];
                if (/^at[\d]?\(([0-9]+),'([^']*)',([0-9]+),([0-9]+),'([^']*)','([^']*)','([^']*)','([^']*)','([^']*)',([0-9]+),([0-9]+),'([^']*)','([^']*)','([^']*)','([^']*)','([^']*)',([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)/) {
                    my @nova;
                 
                    $nova[0]  = $1;  # id 
                    $nova[1]  = $2;  # time
                    $nova[2]  = $3;  # server-id
                    $nova[3]  = $4;  # don't know
                    $nova[4]  = $5;  # don't know
                    $nova[5]  = $6;  # section
                    $nova[6]  = $7;  # link-text
                    $nova[7]  = $8;  # link-url
                    $nova[8]  = $9;  # size
                    $nova[9]  = $10; # seeds
                    $nova[10] = $11; # DLs
                    $nova[11] = $12; # kind
                    $nova[12] = $13; # don't know
                    $nova[13] = $14; # Submitter
                    $nova[14] = $15; # don't know
                    $nova[15] = $16; # info
                    $nova[16] = $17; # don't know
                    $nova[17] = $18; # don't know
                    $nova[18] = $19; # don't know
                    $nova[19] = $20; # don't know 
                    $nova[20] = $21; # don't know
                    $nova[21] = $22; # don't know
                    $nova[22] = $23; # don't know
                    $nova[23] = $24; # don't know
                    $nova[24] = $25; # 1=registered
                    
        		    my $cat_short = $nova[5]; 
		            $cat_short =~ s/^(.*?) - (.*)$/$1/;
              
                    $nova[6] =~ s/\\//g;
                        
                    if ($previous_cat ne $nova[5]) {
                        #----- print out table headline -----
                        if ($count eq 0) { 
                            print OUT '<p class="cat"><a href="index.html">Back to category listing</a>';
                            if ($jumpcount gt 1) { print OUT $jumpto; }
                            print OUT '</p><p class="cat">'.
                                      '<img src="imdb.gif" title="imdb link" alt="imdb" /> = imdb link, '.
                                      '<img src="link.gif" title="generic link" alt="link"/> = generic link, '.
                                      '<img src="reg.gif" title="tracker requires registration" alt="" /> = tracker requires registration, ',
                                      'click on column name to sort';
                            print OUT '</p><table class="torrent"><thead>'; 
                        }
                        printf OUT '</tbody></table><table class="torrent"><thead><tr><td class="torrentheader-typeheader-td" colspan="7">';
                        if ($prev_catshort ne $cat_short) {
                            printf OUT '<a name="%s"></a>', noblanks($cat_short);
                        }
                        printf OUT "%s</td></tr>\n".
                               '<tr><td class="th-dt" onclick="sortTable(\'t%d\',0)" title="click to sort by date">Since'.
                               '<img alt="ns" id="img_t%d_0" src="nosort.gif" /></td>'.
                               '<td class="th-nm"     onclick="sortTable(\'t%d\',1)" title="click to sort by name">Files'.
                               '<img alt="ns" id="img_t%d_1" src="nosort.gif" /></td>'.
                               '<td class="th-fs"     onclick="sortTable(\'t%d\',2)" title="click to sort by size">Filesize'.
                               '<img alt="ns" id="img_t%d_2" src="nosort.gif" /></td>'.
                               '<td class="th-se"     onclick="sortTable(\'t%d\',3)" title="click to sort by seeds">Seeds'.
                               '<img alt="ns" id="img_t%d_3" src="nosort.gif" /></td>'.
                               '<td class="th-le"     onclick="sortTable(\'t%d\',4)" title="click to sort by downloaders">DLs'.
                               '<img alt="ns" id="img_t%d_4" src="nosort.gif" /></td>'.
                               '<td class="th-qa"     onclick="sortTable(\'t%d\',5)" title="click to sort by kind">Kind'.
                               '<img alt="ns" id="img_t%d_5" src="nosort.gif" /></td>'.
                               '<td class="th-sm"     onclick="sortTable(\'t%d\',6)" title="click to sort by submitter">Submitter'.
                               '<img alt="ns" id="img_t%d_6" src="nosort.gif" /></td>'.
                               "</tr></thead>\n<tbody id=\"t%d\">\n",
                               $nova[5],
                               $count, $count, $count, $count, $count, $count, $count,
                               $count, $count, $count, $count, $count, $count, $count,
                               $count;
                        #----- save kategory -----       
                        if ($subcat_id eq 'new') {
                            printf STDOUT "(new) %s\n", $nova[5];
                        } else {
                            printf STDOUT "(%03d) %s\n", $subcat_id, $nova[5];
                            $categorylist{$nova[5]}=$subcat_id;
                        }
                    }
                    $count++;
                    $previous_cat=$nova[5];
                    $prev_catshort=$cat_short;
                     
                    my $info;
                    my $reg;
                                     
                    if ($nova[15] =~ /imdb.com/i) {
                        $info='<a href="' . $nova[15] . '"><img src="imdb.gif" title="imdb link" alt="imdb" /></a>';
                    } elsif ($nova[15] eq "") {
                        $info='<img src="blank.gif" title="" alt="" />';
                    } else {
                        $info='<a href="' . $nova[15] . '"><img src="link.gif" title="generic link" alt="link" /></a>';
                    }
                
                    if ($nova[24] eq '1') {
                        $reg='<img src="reg.gif" title="tracker requires registration" alt="reg" />';
                    } else {
                        $reg='<img src="blank.gif" title="" alt="" />';
                    }
                    
                    
                    printf OUT "<tr><td class='dt'>%s</td>".
                           "<td class='nm'>%s%s<a href=\"get.php?%s\" class=\"ln\">%s</a></td>".
                           "<td class='fs'>%s</td>".
                           "<td class='se%s'>%s</td>".
                           "<td class='le'>%s</td>".
                           "<td class='qa'>%s</td>".
                           "<td class='sm'>%s</td></tr>\n",
                           $nova[1], # date/time
                           $info, $reg, $servers{$nova[2]}."/torrents/".$nova[7], $nova[6], # url
                           $nova[8], # filesize
                           $nova[9] eq "0" ? "-empty" : "", $nova[9], # seeds
                           $nova[10], # leeches
                           $nova[11], # quality
                           $nova[13] eq "" ? "-" : $nova[13];
                }
            }
        }
    }
    printf OUT '</tbody></table></body></html>';
    close(OUT);
}
#---- noblanks --------------------------------------------------------------
sub noblanks {
	my $in = shift || return -1;
	$in =~ s/ /_/g;
	return $in;
}

#---- write_header ----------------------------------------------------------
sub write_header {
    my $filename = shift || return -1;

open(OUT,"> $filename.html") || die("Error: can't open $filename for writing: $!");
print OUT <<EOF;
<?xml version="1.0" encoding="iso-8859-1"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <title>Suprnova light</title>
    <link rel="stylesheet" type="text/css" media="screen" href="snmirror.css" />
    <link rel="shortcut icon" href="favicon.ico" />
    <meta name="robots" content="noindex,nofollow,noarchive" />
    <script src="tablesort.js" type="text/javascript"></script>
</head>
<body>
    <table class="head"><tr><td><p class="title">Suprnova Light</p>
    </td><td><p class="mirror">Created with <a href="http://www.rootdir.de/bt/">snmirror.pl</a> on  
EOF
    print OUT strftime ('%Y-%m-%d, <b>%H:%M:%S %Z</b>', localtime( time() ) );

print OUT <<EOF;
    </p>
    </td></tr></table>
    <p class="note"><b>Note</b>: This is not an official suprnova mirror. We were annoyed by the fact how
    crappy suprnova creates and designs HTML. We just stripped the HTML-files from suprnova to the minimum and
    added compression if the browser supports it. This reduced the filesize by 75%.
    <a href="http://www.rootdir.de/bt/">Read here on how to set up your own suprnova light mirror.</a>
    </p>
EOF
    print OUT $banner;
    
print OUT <<EOF;
<p>Try our <a href="search.php">search function</a>.</p>
EOF
close (OUT);
}


#----  get  -----------------------------------------------------------------
sub get {
    my $uri = shift || return -1; 
    my $refresh = 0;
    
    my $response=doget($uri);
    while ( $response->content =~ /<meta http-equiv="refresh".*?URL=([^'">]+)/i && $refresh < 5 ) {
         #   print "\t\tREFRESH $1\n";
            $response=doget($1);
            $refresh++;
    }
   
    if ($response->is_error($response)) {
    	#print "ERROR: could not download $uri.\n";
	return '';
    }
   
    return $response->content;
}

sub doget {
    my $uri = shift || return -1; 
    
    # print "$uri\n";

    my $request = HTTP::Request->new('GET');
    my $ua = LWP::UserAgent->new();
    $ua->agent('Mozilla/4.76 [en] (Win98; U)');
    $ua->timeout(20);
    if($proxy ne '') {
        $ua->proxy(['http', 'ftp'], $proxy );
    }
    $request->header('Referer', 'http://www.goatpr0n.com/');
    $request->url($uri);
    my $response = $ua->request($request);
    
    my $url = $response->request->url->as_string;
    my $method = $response->request->method;
    my $code = $response->code;
    # printResponseChain($response);

    return $response;
}

sub printResponseChain
{
    my($response) = @_;
    return unless defined $response;
    printResponseChain($response->previous);
    my $method = $response->request->method;
    my $url = $response->request->url->as_string;
    my $code = $response->code;
    print "\t\t$method $url --> ", $response->status_line, "\n";
}
#----------------------------------------------------------------------------
